qdolor.worker-pool.core

Worker pool protocol.

Worker pool protocol.
raw docstring

qdolor.worker-pool.impl.core-async

Worker pool implementation backed by core.async io-threads. Workers run as io-threads (virtual threads on JDK 21+).

Worker pool implementation backed by `core.async` io-threads.
Workers run as io-threads (virtual threads on JDK 21+). 
raw docstring

qdolor.worker-pool.impl.vthreads

Worker pool implementation backed by Java virtual threads (JDK 21+) or a fixed platform thread pool.

Shutdown is coordinated via an atom. stop! blocks on each worker future until it exits its current iteration.

Worker pool implementation backed by Java virtual threads (JDK 21+) or a
fixed platform thread pool.

Shutdown is coordinated via an atom. `stop!` blocks on each worker future
until it exits its current iteration.
raw docstring

cljdoc builds & hosts documentation for Clojure/Script libraries