Worker pool implementation backed by core.async io-threads.
Workers run as io-threads (virtual threads on JDK 21+).
Worker pool implementation backed by `core.async` io-threads. Workers run as io-threads (virtual threads on JDK 21+).
Worker pool implementation backed by Java virtual threads (JDK 21+) or a fixed platform thread pool.
Shutdown is coordinated via an atom. stop! blocks on each worker future
until it exits its current iteration.
Worker pool implementation backed by Java virtual threads (JDK 21+) or a fixed platform thread pool. Shutdown is coordinated via an atom. `stop!` blocks on each worker future until it exits its current iteration.
cljdoc builds & hosts documentation for Clojure/Script libraries
| Ctrl+k | Jump to recent docs |
| ← | Move to previous article |
| → | Move to next article |
| Ctrl+/ | Jump to the search field |